"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.formatTableOfContents = void 0;
/**
 * 목차 구조를 계층적으로 변환하는 함수
 */
const formatTableOfContents = (tableOfContents, reporter) => {
    if (!tableOfContents || tableOfContents.length === 0) {
        return [];
    }
    try {
        // Deep clone to avoid modifying original
        const tocClone = JSON.parse(JSON.stringify(tableOfContents));
        // Ensure TOC entries are sorted by level first, then by appearance order
        const sortedToc = tocClone.sort((a, b) => {
            // Level 기준 정렬
            if ((a.level || 0) !== (b.level || 0)) {
                return (a.level || 0) - (b.level || 0);
            }
            // Original order is preserved
            return 0;
        });
        // Add contextTitle where missing
        for (const entry of sortedToc) {
            if (!entry.contextTitle) {
                entry.contextTitle = entry.title;
            }
        }
        // 1. Find all main headings (h1)
        const mainHeadings = sortedToc.filter((entry) => entry.level === 1);
        // 2. Find all command headings (h2) with their parent h1
        const commandHeadings = sortedToc.filter((entry) => entry.level === 2);
        // Group h2 by parent h1 for easier traversal
        const h1ToH2Map = new Map();
        for (const h1 of mainHeadings) {
            h1ToH2Map.set(h1.hash, []);
        }
        for (const h2 of commandHeadings) {
            if (h2.parentHash) {
                const h2List = h1ToH2Map.get(h2.parentHash) || [];
                h2List.push(h2);
                h1ToH2Map.set(h2.parentHash, h2List);
            }
        }
        // 3. Find all subheadings (h3) with their parent h2
        const subheadings = sortedToc.filter((entry) => entry.level === 3);
        // Group h3 by parent h2 for easier traversal
        const h2ToH3Map = new Map();
        for (const h2 of commandHeadings) {
            h2ToH3Map.set(h2.hash, []);
        }
        for (const h3 of subheadings) {
            if (h3.parentHash) {
                const h3List = h2ToH3Map.get(h3.parentHash) || [];
                h3List.push(h3);
                h2ToH3Map.set(h3.parentHash, h3List);
            }
        }
        // Find all unique h3 titles that appear anywhere
        const allH3Titles = new Set();
        for (const h3 of subheadings) {
            allH3Titles.add(h3.title);
        }
        // Common expected subheadings for commands
        const commonSubheadings = [
            "요약",
            "개요",
            "설명",
            "옵션",
            "문법",
            "주요 명령어",
            "주요 속성 설명",
            "설치",
            "기본 명령",
        ];
        // Add common subheadings to the set
        for (const title of commonSubheadings) {
            allH3Titles.add(title);
        }
        reporter.info(`Found ${allH3Titles.size} unique h3 titles across sections`);
        // For each command, ensure all common subheadings exist
        const finalToc = [...sortedToc];
        let addedEntries = 0;
        // Create a map of known title combinations to avoid duplicates
        const knownCombinations = new Set();
        // Add all existing combinations to the known set
        for (const entry of subheadings) {
            if (entry.parentHash) {
                const parentCommand = commandHeadings.find((h2) => h2.hash === entry.parentHash);
                if (parentCommand) {
                    const combo = `${parentCommand.title}:${entry.title}`;
                    knownCombinations.add(combo);
                }
            }
        }
        // Check which commands are missing which subheadings
        for (const h2 of commandHeadings) {
            const h3Entries = h2ToH3Map.get(h2.hash) || [];
            const existingTitles = new Set(h3Entries.map((entry) => entry.title));
            // For each expected subheading, check if it exists
            for (const title of commonSubheadings) {
                const combo = `${h2.title}:${title}`;
                // Skip if we already have this combination
                if (knownCombinations.has(combo) || existingTitles.has(title)) {
                    continue;
                }
                // We don't have this subheading for this command - but don't add synthetic entries
                // in this implementation as we want to show only what's actually in the document
            }
        }
        // If we added entries, re-sort the TOC
        if (addedEntries > 0) {
            reporter.info(`Added ${addedEntries} missing subheadings to TOC`);
            finalToc.sort((a, b) => {
                if ((a.level || 0) !== (b.level || 0)) {
                    return (a.level || 0) - (b.level || 0);
                }
                return 0;
            });
        }
        // Always make sure all h3 entries have context titles that include their parent command
        for (const entry of finalToc) {
            if (entry.level === 3 && entry.parentHash) {
                const parentCommand = commandHeadings.find((h2) => h2.hash === entry.parentHash);
                if (parentCommand) {
                    entry.contextTitle = `${entry.title} (${parentCommand.title})`;
                }
            }
        }
        return finalToc;
    }
    catch (error) {
        reporter.error(`Error formatting table of contents: ${error}`);
        return tableOfContents; // Return original on error
    }
};
exports.formatTableOfContents = formatTableOfContents;
